use bevy::{asset::LoadedFolder, prelude::*};

#[derive(Debug, Clone, Copy, Default, PartialEq, Eq, Hash, States)]
pub enum AssetLoadState {
    #[default]
    AssetsLoading,
    AssetsLoaded,
}

pub struct AssetLoaderPlugin;

impl Plugin for AssetLoaderPlugin {
    fn build(&self, app: &mut App) {
        app.init_state::<AssetLoadState>()
            .add_systems(OnEnter(AssetLoadState::AssetsLoading), load_assets)
            .add_systems(
                Update,
                check_assets.run_if(in_state(AssetLoadState::AssetsLoading)),
            );
    }
}

#[derive(Resource, Default)]
struct AssetFolder(Handle<LoadedFolder>);

fn load_assets(mut commands: Commands, asset_server: Res<AssetServer>) {
    println!("Loading assets.");
    // load multiple, assets from a folder
    commands.insert_resource(AssetFolder(asset_server.load_folder(".")));
}

fn check_assets(
    mut next_state: ResMut<NextState<AssetLoadState>>,
    rpg_sprite_folder: Res<AssetFolder>,
    mut events: EventReader<AssetEvent<LoadedFolder>>,
) {
    // Advance the `AppState` once all sprite handles have been loaded by the `AssetServer`
    for event in events.read() {
        if event.is_loaded_with_dependencies(&rpg_sprite_folder.0) {
            next_state.set(AssetLoadState::AssetsLoaded);
            println!("Assets loaded.");
        }
    }
}
