use bevy::{prelude::*, window::PrimaryWindow};

use crate::{player::Player, system_set::MovementSet};

const CAM_LERP_FACTOR: f32 = 1000.;

pub struct CameraPlugin;

impl Plugin for CameraPlugin {
    fn build(&self, app: &mut App) {
        app.add_systems(Startup, setup).add_systems(
            Update,
            (move_camera, follow_player).in_set(MovementSet::PostPlayerMovement),
        );
    }
}

fn setup(mut commands: Commands) {
    commands.spawn(Camera2dBundle::default());
}

fn move_camera(
    keys: Res<ButtonInput<KeyCode>>,
    mut camera: Query<&mut Transform, With<Camera2d>>,
    time: Res<Time>,
) {
    let Ok(mut camera) = camera.get_single_mut() else {
        return;
    };

    if keys.pressed(KeyCode::KeyW) {
        camera.translation.y += time.delta_seconds() * CAM_LERP_FACTOR
    }
    if keys.pressed(KeyCode::KeyS) {
        camera.translation.y -= time.delta_seconds() * CAM_LERP_FACTOR
    }
    // if keys.pressed(KeyCode::KeyA) {
    //     camera.translation.x -= time.delta_seconds() * CAM_LERP_FACTOR
    // }
    // if keys.pressed(KeyCode::KeyD) {
    //     camera.translation.x += time.delta_seconds() * CAM_LERP_FACTOR
    // }
}

fn follow_player(
    mut camera: Query<&mut Transform, With<Camera2d>>,
    player: Query<&mut Transform, (With<Player>, Without<Camera2d>)>,
    window: Query<&Window, With<PrimaryWindow>>,
) {
    let window = window.single();

    let Ok(mut camera) = camera.get_single_mut() else {
        return;
    };

    let Ok(player) = player.get_single() else {
        return;
    };

    // Always position the camera so the player is in the first 1/4th of the screen
    camera.translation.x = player.translation.x + window.width() / 4.0;
}
