mod asset_loader;
mod background;
mod camera;
mod obstacle;
mod player;
mod system_set;

use bevy::prelude::*;

fn main() {
    App::new()
        .add_plugins(DefaultPlugins.set(WindowPlugin {
            primary_window: Some(Window {
                title: "Tappy Plane".into(),
                name: Some("TappyPlane.app".into()),
                resolution: (800., 480.).into(),
                ..default()
            }),
            ..default()
        }))
        // .add_plugins(DefaultPlugins.set(ImagePlugin::default_nearest()))
        .add_plugins((
            asset_loader::AssetLoaderPlugin,
            camera::CameraPlugin,
            background::BackgroundPlugin,
            player::PlayerPlugin,
            obstacle::ObstaclePlugin,
            system_set::SystemSetPlugin,
        ))
        .add_systems(Update, quit_on_escape_system)
        .run();
}

fn quit_on_escape_system(
    keys: Res<ButtonInput<KeyCode>>,
    mut app_exit_events: ResMut<Events<AppExit>>,
) {
    if keys.just_pressed(KeyCode::Escape) {
        app_exit_events.send(AppExit::Success);
    }
}
