use bevy::prelude::*;

pub struct SystemSetPlugin;

impl Plugin for SystemSetPlugin {
    fn build(&self, app: &mut App) {
        app.configure_sets(
            Update,
            (
                (
                    MovementSet::PlayerMovement,
                    MovementSet::PostPlayerMovement,
                    MovementSet::PostCameraMovement,
                )
                    .chain(),
                (
                    DespawnSet::DespawnCollectedStars,
                    // TODO flush cmd buffer here
                    DespawnSet::DespawnOffscreen,
                )
                    .chain(),
            )
                .chain(),
        );
        app.add_systems(
            Update,
            apply_deferred
                .after(DespawnSet::DespawnCollectedStars)
                .before(DespawnSet::DespawnOffscreen),
        );
    }
}

#[derive(SystemSet, Debug, Hash, PartialEq, Eq, Clone)]
pub enum MovementSet {
    // Player position is updated here
    PlayerMovement,

    // Systems that check the player position are updated here (such as the camera)
    PostPlayerMovement,

    // Systems that check the camera position are updated here
    PostCameraMovement,
}

#[derive(SystemSet, Debug, Hash, PartialEq, Eq, Clone)]
pub enum DespawnSet {
    DespawnCollectedStars,
    DespawnOffscreen,
}
